<?php
/**
 * Skeleton_Driver:: defines an API for implementing storage backends for Skeleton.
 *
 * $Horde: skeleton/lib/Driver.php,v 1.6 2005/01/03 14:35:32 jan Exp $
 *
 * Copyright 2004-2005 Your Name <you@example.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Your Name <you@example.com>
 * @version $Revision: 1.6 $
 * @since   Skeleton 0.1
 * @package Skeleton
 */
class Skeleton_Driver {

    /**
     * Attempts to return a concrete Skeleton_Driver instance based on $driver.
     *
     * @param string $driver  The type of the concrete Skeleton_Driver subclass
     *                        to return.  The class name is based on the
     *                        storage driver ($driver).  The code is
     *                        dynamically included.
     *
     * @param array $params   A hash containing any additional configuration
     *                        or connection parameters a subclass might need.
     *
     * @return mixed  The newly created concrete Skeleton_Driver instance, or
     *                false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Skeleton_Driver_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete Skeleton_Driver instance
     * based on $driver. It will only create a new instance if no
     * Skeleton_Driver instance with the same parameters currently exists.
     *
     * This method must be invoked as: $var = &Skeleton_Driver::singleton()
     *
     * @param string $driver  The type of concrete Skeleton_Driver subclass to
     *                        return.  The is based on the storage driver
     *                        ($driver).  The code is dynamically included.
     * @param array $params   A hash containing any additional configuration
     *                        or connection parameters a subclass might need.
     *
     * @return mixed  The created concrete Skeleton_Driver instance, or false
     *                on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Skeleton_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
